<?php
/**
 * This file is part of
 * pragmaMx - Web Content Management System.
 * Copyright by pragmaMx Developer Team - http://www.pragmamx.org
 *
 * pragmaMx is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * $Revision: 6 $
 * $Author: PragmaMx $
 * $Date: 2015-07-08 09:07:06 +0200 (Mi, 08. Jul 2015) $
 */

/* Datenbankverbindung aus pragmaMx System verwenden */
include('mainfile.php');

/**
 * pmx_utf8_updater
 *
 * @package pragmaMx
 * @author tora60
 * @copyright Copyright (c) 2013
 * @version $Id: utf8-converter.php 6 2015-07-08 07:07:06Z PragmaMx $
 * @access public
 */
class pmx_utf8_updater {
    private $prefix = '';
    private $user_prefix = '';
    private $language = 'german';

    private $config = array();

    /**
     * pmx_utf8_updater::conf_letters()
     * Das Array mit den zu ersetzenden Sonderzeichen
     *
     * @return
     */
    private function conf_letters()
    {
        $letters = array(/* 'falsch' => 'richtig' */
            'Ð' => 'Ğ',
            'ð' => 'ğ',
            'Ý' => 'İ',
            'ý' => 'ı',
            'Þ' => 'Ş',
            'þ' => 'ş',
            );
        // $letters = array_flip($letters); // zum Umkehren ;-)
        return $letters;
    }

    /**
     * pmx_utf8_updater::conf_tables()
     * bestimmte Tabellen ausschließen
     *
     * @return
     */
    private function conf_tables()
    {
        $excludetable = array (/* mysql LIKE fähig */
            "{$this->prefix}\_captcher",
            "{$this->prefix}\_optimize_gain",
            "{$this->prefix}\_referer",
            "{$this->prefix}\_securelog",
            "{$this->prefix}\_sys_session",
            "{$this->prefix}\_tracking",
            "{$this->prefix}\_visitors",
            /* SMF Forum */
            "{$this->prefix}\_%\_attachments",
            "{$this->prefix}\_%\_log_activity",
            "{$this->prefix}\_%\_log_errors",
            "{$this->prefix}\_%\_log_online",
            "{$this->prefix}\_%\_log_topics",
            "{$this->prefix}\_%\_sessions",
            /* chCounter? */
            "{$this->prefix}\_chc_pages",
            "{$this->prefix}\_chc_referrers",
            "{$this->prefix}\_chc_search_engines",
            );
        return $excludetable;
    }

    /**
     * pmx_utf8_updater::__construct()
     */
    public function __construct()
    {
        $this->config = $GLOBALS['mxConf'];
        $this->prefix = $this->config['prefix'];
        $this->user_prefix = $this->config['user_prefix'];

        /* Die Klasse mit den Sprachkonstanten auswählen */
        $this->language = $GLOBALS['currentlang'];
        if (class_exists('pmx_utf8_updater_' . $this->language)) {
            $this->language = 'pmx_utf8_updater_' . $this->language;
        } else {
            $this->language = 'pmx_utf8_updater_german';
        }

        header('Content-type: text/html; charset=utf-8');

        switch (true) {
            /* nur ein göttlicher Admin darf das ;-)  */
            case !MX_IS_SYSADMIN:
                return mxErrorScreen('Access Denied');
            case empty($_POST['op']):
            default:
                return $this->welcome();
            case $_POST['op'] === 'check':
                return $this->handle(false);
            case $_POST['op'] === 'execute' && empty($_POST['beshure']):
                return $this->handle(false, true);
            case $_POST['op'] === 'execute':
                return $this->handle(true);
        }
    }

    /**
     * pmx_utf8_updater::get_tables()
     *
     * @return array $tables
     */
    private function get_tables()
    {
        /* Tabellen-Array initialisieren */
        $tables = array();

        /* bestimmte Tabellen ausschließen */
        $excludetable = $this->conf_tables();
        $notconv = '';
        foreach ($excludetable as $extable) {
            $notconv .= " AND (`Name` NOT LIKE '{$extable}')";
        }

        /* Alle Tabellen mit $this->prefix und $user_prfix aus DB auslesen.
         * Nicht utf-8 konvertierte und leere Tabellen werden ignoriert
         */
        $result = sql_system_query("SHOW TABLE STATUS
          WHERE `Rows` > 0
            AND `Collation` LIKE 'utf%'
            AND (`Name` LIKE '{$this->prefix}%' OR `Name` LIKE '{$this->user_prefix}%')
            {$notconv}
            ");
        while ($table = sql_fetch_assoc($result)) {
            $tablename = $table['Name'];

            /* Alle Textfelder der Tabelle ermitteln */
            $result2 = sql_system_query("SHOW COLUMNS FROM `{$tablename}`
              WHERE (
                `Type` LIKE '%char%'
                 OR `Type` LIKE '%text%'
                 /* OR `Type` LIKE '%blob%' */
                )");
            while ($column = sql_fetch_assoc($result2)) {
                /* Nur wenn Textfelder gefunden, wird die Tabelle beachtet */
                $tables[$tablename][$column['Field']] = $column['Type'];
            }
        }

        return $tables;
    }

    /**
     * pmx_utf8_updater::welcome()
     *
     * @return
     */
    private function welcome()
    {
        $lang = $this->language;
        ob_start();

        ?>
          <p><?php echo $lang::description ?></p>
          <p><?php echo $lang::theletters ?></p>
          <table class="list">
            <?php foreach ($this->conf_letters() as $old => $new) {

            ?>
              <tr>
                <td class="big"><?php echo $old ?></td>
                <td class="tiny"><?php echo $lang::lettergoto ?></td>
                <td class="big"><?php echo $new ?></td>
              </tr>
            <?php } //endforeach

        ?>
          </table>
          <?php echo $this->form() ?>
        <?php
        $out = ob_get_clean();
        $this->page($out);
    }

    /**
     * pmx_utf8_updater::handle()
     *
     * @param mixed $char
     * @return
     */
    private function handle($execute = false, $error = false)
    {
        /* das kann alles sehr lange dauern ;-)) */
        set_time_limit(0);

        /* Änderungen nur durchführen wenn $execute = TRUE und kein Fehler */
        if ($error) {
            $execute = false;
        }

        /* Tabellen-Array initialisieren */
        $tables = $this->get_tables();

        /* Das Array mit den zu ersetzenden Sonderzeichen */
        $letters = $this->conf_letters();

        /* Die Bedingung für die sql-Abfrage erstellen, so dass nur
         * Datensätze bzw. Felder gefunden werden, welche die
         * gesuchten falschen Sonderzeichen enthalten.
         */
        $search = array();
        foreach ($letters as $char => $replchar) {
            /* Hexadezimale Schreibweise der Sonderzeichen für mySql generieren */
            $char = $this->uni2mhex($char);
            $replchar = $this->uni2mhex($replchar);
            /* nur INSTR() ist multibyte Sicher für Vergleiche! */
            $search[] = " INSTR(`{FIELDNAME}`, char({$char}))";
            $replace[] = "REPLACE({FIELDNAME},char({$char}),char({$replchar}))";
        }

        /* das verschachtelte REPLACE() für mysql erstellen */
        $new = '';
        foreach ($replace as $str) {
            if (!$new) {
                $new = $str;
            } else {
                $new = str_replace('{FIELDNAME}', $str, $new);
            }
        }

        $allrows = 0;
        $upshot = array();
        foreach ($tables as $tablename => $fields) {
            $numrows = 0;
            foreach ($fields as $field => $type) {
                $where = "(`{$field}`<>'') AND (" . str_replace('{FIELDNAME}', $field, implode(' OR ', $search)) . ")";
                $valuetoset = str_replace('{FIELDNAME}', "`{$field}`", $new);
                $limit = ($execute) ? " LIMIT 1" : "";
                $qry = "SELECT `{$field}`
                  FROM `{$tablename}`
                  WHERE {$where} AND `{$field}` <> {$valuetoset}
                  {$limit}";
                $result = sql_system_query($qry);
                $numrows = sql_num_rows($result);
                if (!$execute && $numrows) {
                    $allrows += $numrows;
                    $upshot[$tablename][$field] = $numrows;
                    continue;
                }
                if ($numrows) {
                    $qry2 = "UPDATE `{$tablename}`
                      SET `{$field}` = {$valuetoset}
                      WHERE {$where} AND `{$field}` <> {$valuetoset}
                      ";
                    sql_system_query($qry2);
                    $numrows = sql_affected_rows();
                    $allrows += $numrows;
                    $upshot[$tablename][$field] = $numrows;
                }
            }
            if ($execute && $numrows) {
                sql_system_query("OPTIMIZE TABLE `{$tablename}`");
            }
        }

        $lang = $this->language;

        if (!$allrows) {
            $out = '<p class="success align-left">' . $lang::resultcheck1 . '</p>
                    <p><a href="' . $_SERVER['PHP_SELF'] . '">' . $lang::goback . '</a></p>';
            return $this->page($out);
        }

        if ($execute) {
            $message = $lang::resultexec;
            $class = 'success';
        } else {
            $message = $lang::resultcheck2;
            $class = 'info';
        }

        ob_start();

        ?>
        <p class="<?php echo $class ?> align-left"><?php echo $message ?></p>
        <table class="list">
          <tr>
            <th colspan="2"><?php echo $lang::resulttable ?></th>
          </tr>
          <tr>
            <td><?php echo $lang::resultfield ?></td>
            <td><?php echo $lang::resultrows ?></td>
          </tr>
          <?php foreach ($upshot as $tablename => $fields) {

            ?>
            <tr>
              <th colspan="2"><?php echo $tablename ?></th>
            </tr>
            <?php foreach ($fields as $field => $cnt) {

                ?>
              <tr>
                <td><?php echo $field ?></td>
                <td class="align-right"><?php echo $cnt?></td>
              </tr>
          <?php }
        }

        ?>
          <tr>
            <th><?php echo $lang::resultall ?></th>
            <td class="align-right"><b><?php echo $allrows ?></b></td>
          </tr>
        </table>
        <?php echo $this->form(true, $error) ?>
        <?php
        $out = ob_get_clean();
        return $this->page($out);
    }

    /**
     * pmx_utf8_updater::form()
     *
     * @return
     */
    private function form($goback = false, $error = false)
    {
        $lang = $this->language;
        ob_start();

        ?>
        <?php if ($goback) {

            ?>
        <p><a href="<?php echo $_SERVER['PHP_SELF'] ?>"><?php echo $lang::goback ?></a></p>
        <?php } else {

            ?>
        <p><?php echo $lang::waiting ?></p>
        <form method="post" action="<?php echo $_SERVER['PHP_SELF'] ?>">
          <p>
            <input type="hidden" name="op" value="check">
            <input type="submit" value="<?php echo $lang::onlycheck ?>" />
          </p>
        </form>
        <?php } //endif

        ?>
        <br /><br />
        <form method="post" action="<?php echo $_SERVER['PHP_SELF'] ?>">
          <p>
            <?php echo $lang::backupbeshure ?><br />
            <?php if ($error) {

            ?>
              <span class="error"><?php echo $lang::backupbeshureok ?></span><br />
            <?php } //endif

        ?>
            <input type="hidden" name="op" value="execute">
            <input type="submit" value="<?php echo $lang::execute ?>" />&nbsp;
            <input type="checkbox" name="beshure" value="1" title="<?php echo $lang::backupbeshureok ?>" />&nbsp;<?php echo $lang::backupbeshureyes ?>
          </p>
        </form>
      <?php
        return ob_get_clean();
    }

    /**
     * pmx_utf8_updater::page()
     *
     * @param mixed $out
     * @return
     */
    private function page($out)
    {
        include('header.php');
        echo '<h2>pragmaMx ' . PMX_VERSION . ', UTF-8 corrector</h2>';
        echo $out;
        include('footer.php');
    }

    /**
     * pmx_utf8_updater::uni2mhex()
     * Hexadezimale Schreibweise der Sonderzeichen für mySql generieren
     *
     * @param mixed $char
     * @return
     */
    private function uni2mhex($char)
    {
        $arr = str_split($char);
        $char = '0x';
        foreach ($arr as $part) {
            // z.B. Das Zeichen ý ergibt so char(0xc3bd)
            $char .= dechex(ord($part));
        }
        return $char;
    }
}

/**
 * pmx_utf8_updater_german
 * Deutsche Sprachkonstanten (Sie-Form)
 */
class pmx_utf8_updater_german {
    // const constant = 'Konstanter Wert';
    const description = 'Beim Update von pragmaMx 1.x, kann es vorkommen, daß bestimmte Sonderzeichen in den Datenbanktabellen nicht korrekt in das neue UTF-8 Format konvertiert werden. Mit diesem Script können Sie prüfen ob Ihre Datenbanktabellen betroffen sind und diese gegebenenfalls korrigieren lassen. Normalerweise sind nur türkische Sonderzeichen betroffen.';
    const theletters = 'Folgende Sonderzeichen werden umgeschrieben:';
    const lettergoto = 'wird zu:';
    const onlycheck = 'Tabellen prüfen';
    const execute = 'Tabellen ändern';
    const backupbeshure = 'Stellen Sie vor der Änderung der Datenbanktabellen bitte sicher, daß Sie eine aktuelle Sicherung der Datenbank haben.';
    const backupbeshureyes = 'Ja, ich habe eine aktuelle Datenbanksicherung.';
    const backupbeshureok = 'Bestätigen Sie bitte, dass Sie eine aktuelle Datenbanksicherung haben.';
    const goback = 'Zurück';
    const resulttable = 'Tabelle';
    const resultfield = 'Feldname';
    const resultrows = 'geänderte Zeilen';
    const resultall = 'insgesamt:';
    const resultcheck1 = 'Bei der Überprüfung der Datenbanktabellen wurden keine zu korrigierende Sonderzeichen gefunden.';
    const resultcheck2 = 'Bei der Überprüfung wurden zu korrigierende Sonderzeichen gefunden. <br />Folgende Datenbanktabellen sind betroffen:';
    const resultexec = 'Es wurden folgende Änderungen in den Datenbanktabellen vorgenommen:';
    const waiting = 'Die Überprüfung sowie das Ändern der Datenbanktabellen kann länger dauern, bitte haben Sie etwas Geduld.';
}

/**
 * pmx_utf8_updater_german_du
 * Deutsche Sprachkonstanten (Du-Form)
 */
class pmx_utf8_updater_german_du extends pmx_utf8_updater_german {
    const description = 'Beim Update von pragmaMx 1.x, kann es vorkommen, daß bestimmte Sonderzeichen in den Datenbanktabellen nicht korrekt in das neue UTF-8 Format konvertiert werden. Mit diesem Script kannst du prüfen ob deine Datenbanktabellen betroffen sind und diese gegebenenfalls korrigieren lassen. Normalerweise sind nur türkische Sonderzeichen betroffen.';
    const backupbeshure = 'Stelle vor der Änderung der Datenbanktabellen bitte sicher, daß du eine aktuelle Sicherung der Datenbank hast.';
    const backupbeshureok = 'Bestätige bitte, dass du eine aktuelle Datenbanksicherung hast.';
    const waiting = 'Die Prüfung sowie das Ändern der Datenbanktabellen kann länger dauern, bitte habe etwas Geduld.';
}

/**
 * pmx_utf8_updater_turkish
 * Türkische Sprachkonstanten
 */
class pmx_utf8_updater_turkish extends pmx_utf8_updater_german {
	const description = 'pragmaMx 1.x güncelleştirmesi sırasında, veritabanı tablolarında belirli özel karakterler yeni UTF-8 biçimine doğru dönüştürülmeyebilir. Bu komut ile veritabanı tabloların etkilenir olup olmadığını kontrol edebilir ve gerekirse düzeltilebilirsiniz. Normalde tek Türk özel karakterler etkilenir.';
    const theletters = 'Aşağıdaki özel karakterler yeniden dönüştürülüyor:';
    const lettergoto = 'olacağı:';
    const onlycheck = 'Tabloları kontrol et';
    const execute = 'Tabloları değiştir';
    const backupbeshure = 'Veritabanı tabloları değiştirmeden önce, güncel bir veritabanı yedeğine sahip olduğunuzdan emin olun.';
    const backupbeshureyes = 'Evet, güncel bir veritabanı yedeklemesi var.';
    const backupbeshureok = 'Geçerli bir veritabanı yedeği olduğunu teyit edin.';
    const goback = 'Geri';
    const resulttable = 'Tablo';
    const resultfield = 'Alan adı';
    const resultrows = 'değiştirilen satırlar';
    const resultall = 'toplam:';
    const resultcheck1 = 'Düzeltilmesi gereken veritabanı tabloları incelerken hiçbir özel karakter bulunamadı.';
    const resultcheck2 = 'İncelemede özel karakterlerin düzeltilmesi bulundu. <br />Aşağıdaki veritabanı tabloları etkilenir:';
    const resultexec = 'Veritabanı tablolarında aşağıdaki değişiklikler yapılmıştır:';
    const waiting = 'İnceleme ve veritabanı tabloların değiştirilmesi uzun sürebilir, lütfen sabırlı olun.';
}

$uu = new pmx_utf8_updater();
$uu = null;

?>